-- ====================================================================
--                 PROCEDIMENTO PARA POSIO DE ESTOQUE
--                 ------------ ---- ------- -- -------
--
--  PARMETROS: P_FILIAL     = CDIGO DA FILIAL A SER CONSIDERADA.          (IN)
--  ----------  P_PRODUTO    = PRODUTO A SER ATUALIZADO.                    (IN)
--              P_DTPOSICAO  = DATA INICIAL PARA CLCULO DO ESTOQUE.        (IN)
--              P_QTDEESTO   = QUANTIDADE EM ESTOQUE.                       (OUT)
--              P_VALRESTO   = VALOR DO ESTOQUE.                            (OUT)
--              P_VALRBRUT   = VALOR DO ESTOQUE (BRUTO).                    (OUT)
--              P_POSNZERO   = VERIFICA LTIMA POSIO ANTES DE ZERAR.      (IN) DEFAULT 'N'
--              P_ALMOXARIFE = CDIGO DO ALMOXARIFADO A SER CONSIDERADO.    (IN) DEFAULT NULL
--              P_LOCALESTOQ = CDIGO DO LOCAL DE ESTOQUE A SER CONSIDERADO.(IN) DEFAULT NULL
--
-- @C:\Web_405\SQL\END_CONV\ORACLE\0013_GERSTPo_POSICAO_ESTOQUE.SQL
--
-- ====================================================================

CREATE OR REPLACE PROCEDURE POSICAO_ESTOQUE(
   P_FILIAL       IN POSESTDIA.CDFILIAL%TYPE,
   P_PRODUTO      IN POSESTDIA.CDPRODUTO%TYPE,
   P_DTPOSICAO    IN POSESTDIA.DTPOSIESTQ%TYPE,
   P_QTDEESTO     OUT NUMBER,
   P_VALRESTO     OUT NUMBER,
   P_VALRBRUT     OUT NUMBER,
   P_POSNZERO     IN POSESTDIA.IDPOSIZERO%TYPE DEFAULT 'N',
   P_ALMOXARIFE   IN ALMOXARIFADO.CDALMOXARIFE%TYPE DEFAULT NULL,
   P_LOCALESTOQ   IN LOCALESTOQUE.CDLOCALESTOQ%TYPE DEFAULT NULL,
   P_IDUTILALMOX  IN POSESTDIA.IDPOSIZERO%TYPE DEFAULT NULL,
   P_IDUTILLCESTQ IN POSESTDIA.IDPOSIZERO%TYPE DEFAULT NULL,
   P_IDUTILLOTE   IN POSESTDIA.IDPOSIZERO%TYPE DEFAULT NULL,
   P_NRLOTEESTQ   IN POSESTDIA.NRLOTEESTQ%TYPE DEFAULT NULL,
   P_NRSUBLOTE    IN POSESTDIA.NRSUBLOTE%TYPE  DEFAULT NULL)

AS

   V_QTDEESTO     NUMBER;
   V_VALRESTO     NUMBER;
   V_VALRBRUT     NUMBER;
   V_QTDEANTE     NUMBER;
   V_VALRANTE     NUMBER;
   V_VRBRANTE     NUMBER;
   V_QTDEESTOAUX  NUMBER;
   V_VALRESTOAUX  NUMBER;
   V_VALRBRUTAUX  NUMBER;

   V_IDUTILALMOX  VARCHAR2(01);
   V_IDUTILLCESTQ VARCHAR2(01);
   V_IDUTILLOTE   VARCHAR2(01);

   V_ALMOXARIFE   VARCHAR2(06);
   V_LOCALESTOQ   VARCHAR2(06);
   V_LOTEESTQ     VARCHAR2(20);
   V_SUBLOTE      VARCHAR2(04);

BEGIN

   IF P_IDUTILALMOX IS NULL THEN
      SELECT NVL(MAX(IDUTILALMOX),'N'), NVL(MAX(IDUTILLCESTQ),'N'), NVL(MAX(IDUTILLOTE),'N')
      INTO V_IDUTILALMOX, V_IDUTILLCESTQ, V_IDUTILLOTE
      FROM PARAMFILIAL
      WHERE CDFILIAL = P_FILIAL;
   ELSE
      V_IDUTILALMOX  := P_IDUTILALMOX;
      V_IDUTILLCESTQ := P_IDUTILLCESTQ;
      V_IDUTILLOTE   := P_IDUTILLOTE;
   END IF;

   IF P_ALMOXARIFE IS NOT NULL THEN

      V_ALMOXARIFE := P_ALMOXARIFE;
      V_LOCALESTOQ := P_LOCALESTOQ;

      IF P_NRLOTEESTQ IS NULL THEN
         CONSULTA_LOTE(V_IDUTILLOTE, P_FILIAL, V_ALMOXARIFE,
                       V_LOCALESTOQ, P_PRODUTO,
                       V_LOTEESTQ, V_SUBLOTE);
      ELSE
         V_LOTEESTQ := P_NRLOTEESTQ;
         V_SUBLOTE  := P_NRSUBLOTE;
      END IF;

      CONSULTA_POSICAO(P_FILIAL, P_PRODUTO, P_DTPOSICAO,
                       V_ALMOXARIFE, V_LOCALESTOQ, V_LOTEESTQ,
                       V_SUBLOTE, P_POSNZERO,
                       P_QTDEESTO, P_VALRESTO, P_VALRBRUT);
   ELSE
      IF (V_IDUTILLCESTQ = 'N') AND (V_IDUTILALMOX = 'N') THEN

         V_ALMOXARIFE := ' ';
         V_LOCALESTOQ := ' ';

         IF P_NRLOTEESTQ IS NULL THEN
            CONSULTA_LOTE(V_IDUTILLOTE, P_FILIAL, V_ALMOXARIFE,
                          V_LOCALESTOQ, P_PRODUTO,
                          V_LOTEESTQ, V_SUBLOTE);
         ELSE
            V_LOTEESTQ := P_NRLOTEESTQ;
            V_SUBLOTE  := P_NRSUBLOTE;
         END IF;

         CONSULTA_POSICAO(P_FILIAL, P_PRODUTO, P_DTPOSICAO,
                          V_ALMOXARIFE, V_LOCALESTOQ, V_LOTEESTQ,
                          V_SUBLOTE, P_POSNZERO,
                          V_QTDEESTOAUX, V_VALRESTOAUX, V_VALRBRUTAUX);

         P_QTDEESTO := V_QTDEESTOAUX;
         P_VALRESTO := V_VALRESTOAUX;
         P_VALRBRUT := V_VALRBRUTAUX;
      ELSE

         V_QTDEESTOAUX := 0;
         V_VALRESTOAUX := 0;
         V_VALRBRUTAUX := 0;

         FOR C_LOCALIZACAO IN (SELECT CDALMOXARIFE, CDLOCALESTOQ
                               FROM LOCALESTOQUE
                               WHERE CDFILIAL = P_FILIAL) LOOP

            V_ALMOXARIFE := C_LOCALIZACAO.CDALMOXARIFE;
            V_LOCALESTOQ := C_LOCALIZACAO.CDLOCALESTOQ;

            IF P_NRLOTEESTQ IS NULL THEN
               CONSULTA_LOTE(V_IDUTILLOTE, P_FILIAL, V_ALMOXARIFE,
                             V_LOCALESTOQ, P_PRODUTO,
                             V_LOTEESTQ, V_SUBLOTE);
            ELSE
               V_LOTEESTQ := P_NRLOTEESTQ;
               V_SUBLOTE  := P_NRSUBLOTE;
            END IF;

            CONSULTA_POSICAO(P_FILIAL, P_PRODUTO, P_DTPOSICAO,
                             V_ALMOXARIFE, V_LOCALESTOQ, V_LOTEESTQ,
                             V_SUBLOTE, P_POSNZERO,
                             V_QTDEESTO, V_VALRESTO, V_VALRBRUT);
            V_QTDEESTOAUX := V_QTDEESTOAUX + V_QTDEESTO;
            V_VALRESTOAUX := V_VALRESTOAUX + V_VALRESTO;
            V_VALRBRUTAUX := V_VALRBRUTAUX + V_VALRBRUT;
         END LOOP;

         P_QTDEESTO := V_QTDEESTOAUX;
         P_VALRESTO := V_VALRESTOAUX;
         P_VALRBRUT := V_VALRBRUTAUX;
      END IF;
   END IF;
END;
/
